import java.util.*;


/**
 * Erstellen einer SiteMap fr NanoCms
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class SiteMap
{
    private static final String WRK_DIR = "C:/nanocms/cms_dest";

    private static HashSet visitedFilesSet = new HashSet();

    private static HashSet entirePageSet = new HashSet();

    // private static ArrayList entirePageList = new ArrayList();
    private static Page indexPage;

    public static void main(String[] pArgs)
    {
        String indexHtmlFileName = WRK_DIR + "/index.html";

        // Vorgehenweise Breite zuerst
        Link indexHtmlLink = new Link();

        indexHtmlLink.absolutName = indexHtmlFileName;
        indexPage = makePage(indexHtmlFileName, indexHtmlLink/* , 0*/);
        entirePageSet.add(indexHtmlFileName);

        indexPage.subPageArr = processFile(indexHtmlFileName, 0);
        System.out.println(indexPage);
        generateSitemap();
        System.out.println("Anzahl Pages: " + entirePageSet.size());
    }

    /**
     *
     */
    private static void generateSitemap()
    {
        StringBuffer strBuff = new StringBuffer();

        strBuff.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
        strBuff.append("<HTML>\n");
        strBuff.append("<HEAD>\n");
        strBuff.append("<TITLE>SiteMap Xxxx Yyyyy</TITLE>\n");
        // strBuff.append("<BODY BGCOLOR=\"black\">\n");
        strBuff.append("<BODY>\n");

        strBuff.append(generateSubSitemap(indexPage, 0));

        strBuff.append("</BODY>\n");
        strBuff.append("</HTML>\n");
        System.out.println(strBuff);
        StringUtil.str2file(strBuff.toString(), WRK_DIR + "/sitemap.html");
    }

    private static String generateSubSitemap(// --
            final Page pPage,
            final int pDepth)
    {
        StringBuffer strBuff = new StringBuffer();

        // strBuff.append( StringUtil.repeat("&nbsp;", pDepth * 3 ) );
        strBuff.append("<ul>\n");
        strBuff.append("<li><a href=\"" + pPage.hrefStr + "\">");
        strBuff.append(pPage.titleTxt);
        // strBuff.append( "</a><br/>\n");
        strBuff.append("</a></li>\n");

        for (int i = 0; i < pPage.subPageArr.length; i++)
        {
            strBuff.append(generateSubSitemap(pPage.subPageArr[ i ], pDepth + 1));
        }
        strBuff.append("</ul>\n");
        return strBuff.toString();
    }

    /**
     *
     *
     * @param pFileNameWithPath
     * @param pDepth
     */
    private static Page[] processFile(// --
            final String pFileNameWithPath, // --
            final int pDepth)
    {
        if (visitedFilesSet.contains(pFileNameWithPath))
        {
            // schon abgearbeitet
            return new Page[0];
        }

        System.out.println("processFile: " + pFileNameWithPath + " " + pDepth);

        visitedFilesSet.add(pFileNameWithPath);

        Link[] linkArr = getLinkArr(pFileNameWithPath);

        ArrayList pageList = new ArrayList();

        for (int i = 0; i < linkArr.length; i++)
        {
            // Vorgehensweise Breite zuerst
            // final String linkAbsPathAndName = rel2absPath(linkArr[i].hrefStr, pFileNameWithPath);
            final String linkAbsPathAndName = linkArr[i].absolutName;

            if (!entirePageSet.contains(linkAbsPathAndName))
            {
                pageList.add(makePage(linkAbsPathAndName, linkArr[i]/* , pDepth + 1*/));
                entirePageSet.add(linkAbsPathAndName);
            }
        }
        Page[] pageArr = (Page[]) pageList.toArray(new Page[ pageList.size() ]);

        // for (int i = 0; i < linkArr.length; i++)
        // {
        // final String linkAbsPathAndName = rel2absPath(linkArr[i].hrefStr);
        // pageArr[i].subPageArr = processFile(linkAbsPathAndName, pDepth + 1);
        // }
        for (int i = 0; i < pageArr.length; i++)
        {
            // final String linkAbsPathAndName = rel2absPath(pageArr[i].hrefStr , pFileNameWithPath);

            // ??? final String linkAbsPathAndName = linkArr[i].absolutName;
            // final String linkAbsPathAndName = rel2absPath( pageArr[i].hrefStr , pFileNameWithPath);
            final String linkAbsPathAndName = pageArr[i].link.absolutName;

            // System.out.println( "linkAbsPathAndName: " + linkAbsPathAndName );
            pageArr[i].subPageArr = processFile(linkAbsPathAndName, pDepth + 1);
            // System.out.println("pageArr[" + i + "]: " + pageArr[i]);
        }
        return pageArr;
    }

    /**
     * Erzeugen Page-Object
     *
     * @param pFileNameWithPath
     * @return
     */
    private static Page makePage(// --
            final String pFileNameWithPath, // --
            final Link pLink)
    // final int pDepth)
    {
        Page page = new Page();

        page.hrefStr = abs2relPath(pFileNameWithPath);

        String titleTxt = readTitle(pFileNameWithPath);

        titleTxt = StringUtil.cutRight(titleTxt, "Xxxx Yyyyy").trim();
        page.titleTxt = titleTxt;
        page.link = pLink;
        // page.depth = pDepth;
        return page;
    }

    /**
     * Lesen Inhalt des title-Tag
     *
     * @param pFileNameWithPath
     * @return
     */
    private static String readTitle(String pFileNameWithPath)
    {
        // System.out.println("readTitle: " + pFileNameWithPath);
        String contentStr = StringUtil.file2str(pFileNameWithPath);

        // System.out.println( "contentStr" + contentStr );

        // Pattern p = Pattern.compile( "<TITLE>(.*)</TITLE>" , Pattern.CASE_INSENSITIVE );
        // Matcher m = p.matcher(contentStr);
        // if ( ! m.matches())
        // {
        // return ""; // nix gefunden
        // }
        // String titleStr = m.group(1);
        // System.out.println( "tileStr: " +titleStr );
        // return titleStr;

        String startTag = "<title>";
        String endTag = "</title>";
        String lowerContent = contentStr.toLowerCase();
        int fromPos = lowerContent.indexOf(startTag) + startTag.length();
        int toPos = lowerContent.indexOf(endTag);

        if (fromPos < 0 || toPos < 0 || toPos <= fromPos)
        {
            return "";
        }
        String titleStr = contentStr.substring(fromPos, toPos);

        // System.out.println("title: " + titleStr);
        return titleStr;
    }

    /**
     * relativen in absoluten Pfad umwandeln
     *
     * @param pStr relative Angabe
     * @return absolute Angabe
     */
    private static String rel2absPath(// --
            final String pRelativName,
            final String pParentAbsolutName)
    {
        // System.out.println( "pRelativName: " + pRelativName );
        // System.out.println( "pParentAbsolutName: " + pParentAbsolutName );

        if (pRelativName.startsWith("./"))
        {
            // return WRK_DIR + pRelativName.substring(1);
            return getAbsolutPath(pParentAbsolutName) + pRelativName.substring(1);
        }
        if (pRelativName.startsWith("../"))
        {
            return getAbsolutPrePath(pParentAbsolutName) + pRelativName.substring(2);
        }
        // throw new IllegalArgumentException(pRelativName);
        return pRelativName;
    }

    /**
     * @param pParentAbsolutName
     * @return
     */
    private static String getAbsolutPath(String pParentAbsolutName)
    {
        return pParentAbsolutName.substring(0, pParentAbsolutName.lastIndexOf("/"));
    }

    /**
     * @param pParentAbsolutName
     * @return
     */
    private static String getAbsolutPrePath(String pParentAbsolutName)
    {
        return getAbsolutPath(getAbsolutPath(pParentAbsolutName));
    }

    /**
     * absoluten in relativen Pfad umwandeln
     *
     * @param pStr absolute Angabe
     * @return relative Angabe
     */
    private static String abs2relPath(String pStr)
    {
        // System.out.println( "abs2relPath: " + pStr );
        return "." + StringUtil.cutLeft(pStr, WRK_DIR);
    }

    /**
     * @param pFileNameWithPath
     * @return
     */
    private static Link[] getLinkArr(String pFileNameWithPath)
    {
        // System.out.println( "getLinkArr: " + pFileNameWithPath + "-------------------------------------------------------------------------------------");
        String contentStr = StringUtil.file2str(pFileNameWithPath);

        contentStr = stripHtmlComments(contentStr);
        String[] linkStrArr = getLinkStringArr(contentStr);
        Link[] allLinkArr = parseLinkArr(linkStrArr, pFileNameWithPath);
        Link[] filteredLinkArr = filterLinkArr(allLinkArr);

        return filteredLinkArr;
    }

    /**
     * @param contentStr
     * @return
     */
    private static String stripHtmlComments(String pStr)
    {
        StringBuffer strBuff = new StringBuffer();
        int pos = 0;

        while (pos < pStr.length())
        {
            // "<!-- " suchen
            if (StringUtil.charAt(pStr, pos) == '<' && StringUtil.charAt(pStr, pos + 1) == '!'
                    && StringUtil.charAt(pStr, pos + 2) == '-' && StringUtil.charAt(pStr, pos + 3) == '-')
            {
                // bis "-->" laufen
                while (pos < pStr.length()
                        && !(StringUtil.charAt(pStr, pos - 3) == '-'
                                && Character.toLowerCase(StringUtil.charAt(pStr, pos - 2)) == '-'
                                && StringUtil.charAt(pStr, pos - 1) == '>'))
                {
                    pos++;
                }
            }
            strBuff.append(pStr.charAt(pos));
            pos++;
        }

        return strBuff.toString();
    }

    /**
     * Herausfiltern innere Links
     *
     * @param allLinkArr
     * @return
     */
    private static Link[] filterLinkArr(Link[] pLinkArr)
    {
        ArrayList list = new ArrayList();

        for (int i = 0; i < pLinkArr.length; i++)
        {
            if (pLinkArr[i].hrefStr.startsWith(".") // innerer Link
                    && pLinkArr[i].hrefStr.endsWith(".html") // nur HTML-Seiten (keine ZIP)
                    && !pLinkArr[i].hrefStr.startsWith("./sitemap") // nicht die sitemap
                    // && pLinkArr[i].hrefStr.indexOf('/', 3) < 0) //kein Unterverzeichnis
                    && !pLinkArr[i].hrefStr.startsWith("./artikelsys") // nicht Artikelsystematik
                    )
            {
                list.add(pLinkArr[i]);
            }
        }
        return (Link[]) list.toArray(new Link[list.size()]);
    }

    /**
     * Herausparsen der &lt;a ... &lt;/a&gt; - Bereiche
     * aus dem Datei-Inhalt.
     *
     * @param pFileContent
     * @return
     */
    private static String[] getLinkStringArr(String pFileContent)
    {
        int pos = 0;
        ArrayList linkStrList = new ArrayList();

        while (pos < pFileContent.length())
        {
            // "<a " suchen
            if (StringUtil.charAt(pFileContent, pos) == '<'
                    && Character.toLowerCase(StringUtil.charAt(pFileContent, pos + 1)) == 'a'
                    && Character.isWhitespace(StringUtil.charAt(pFileContent, pos + 2))
                    && Character.toLowerCase(StringUtil.charAt(pFileContent, pos + 3)) == 'h')
            // href und nicht name
            {
                StringBuffer strBuff = new StringBuffer();

                // bis "</a>" laufen
                while (pos < pFileContent.length()
                        && !(StringUtil.charAt(pFileContent, pos - 4) == '<'
                                && StringUtil.charAt(pFileContent, pos - 3) == '/'
                                && Character.toLowerCase(StringUtil.charAt(pFileContent, pos - 2)) == 'a'
                                && StringUtil.charAt(pFileContent, pos - 1) == '>'))
                {
                    strBuff.append(StringUtil.charAt(pFileContent, pos));
                    pos++;
                }
                // Link-String vermerken
                linkStrList.add(strBuff.toString());
                // System.out.println(strBuff);
            }

            pos++;
        }
        return (String[]) linkStrList.toArray(new String[linkStrList.size()]);
    }

    /**
     * Erzeugen eines Arrays mit {@link Link}s.
     *
     * @param pLinkStrArr
     * @return
     */
    private static Link[] parseLinkArr(String[] pLinkStrArr, String pFileNameWithPath)
    {
        ArrayList linkList = new ArrayList();

        for (int i = 0; i < pLinkStrArr.length; i++)
        {
            Link link = parseLink(pLinkStrArr[i], pFileNameWithPath);

            // System.out.println( link );
            linkList.add(link);
        }
        return (Link[]) linkList.toArray(new Link[linkList.size()]);
    }

    private static Link parseLink(String pRunLinkStr, String pFileNameWithPath)
    {
        Link link = new Link();

        link.hrefStr = readHref(pRunLinkStr);
        link.absolutName = rel2absPath(link.hrefStr, pFileNameWithPath);
        link.linkTxt = readLinkTxt(pRunLinkStr);
        return link;
    }

    /**
     * @param string
     * @return
     */
    private static String readHref(String pStr)
    {
        int hrefPos = pStr.toLowerCase().indexOf("href=");

        hrefPos += 6;

        String hrefStr = pStr.substring(hrefPos, pStr.indexOf("\"", hrefPos));

        return hrefStr;
    }

    /**
     * @param string
     * @return
     */
    private static String readLinkTxt(String pStr)
    {
        int fromPos = pStr.indexOf("\">", 10) + 2;
        String linkTxt = pStr.substring(fromPos, pStr.length() - 4);

        return linkTxt;
    }

}


class Link
{
    String absolutName;
    String hrefStr;
    String linkTxt;

    public String toString()
    {
        return "Link[absolutName: '" + absolutName + "', hrefStr: '" + hrefStr + "' , linkTxt: '" + linkTxt + "']";
    }
}


class Page
{
    Link link;
    String hrefStr;
    String titleTxt;
    // int depth;
    Page[] subPageArr;

    public String toString()
    {
        return "Page[link" + link + ", hrefStr: '" + hrefStr + "' , titleTxt: '" + titleTxt // + "' , depth: "
                // + depth
                + "' , subPageArr: " + subPageArrToString(/* 0*/) + " ]";
    }

    private String subPageArrToString( // --
            // final int pDepth
            )
    {
        if (subPageArr == null)
        {
            return null;
        }
        StringBuffer strBuff = new StringBuffer("[");

        for (int i = 0; i < subPageArr.length; i++)
        {
            strBuff.append(subPageArr[ i ].toString(/* pDepth + 1*/));
            if (i < subPageArr.length - 1)
            {
                strBuff.append(", ");
            }
        }
        strBuff.append("]");
        return strBuff.toString();
    }
}
