//package de.kuehei.util ;

import java.text.*;
import java.util.*;
import java.io.*;

/**
 * Alle mglichen Methoden, hauptschlich fr Strings . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public final class StringUtil {
  /**
   * Datumsformat fr {@link #date2str} und {@link #parseDate}
   */
  private static SimpleDateFormat formatDate = new SimpleDateFormat("dd.MM.yyyy");

  /**
   * Timestamp-Datumsformat fr {@link #getTimeStamp}
   */
  private static SimpleDateFormat formatTimestamp = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss", Locale.GERMAN);

  /**
   * Constructor declaration<br/>
   * Instanziieren nicht erlaubt
   */
  private StringUtil() { ; }

  /**
   * String auf null, Lnge 0 oder nur Spaces prfen
   *
   * @param strValue zu prfender String
   * @return ob String empty (ja/nein)
   */
  public static final boolean empty(String strValue)
  {
    boolean bReturn = true;

    if (strValue != null)
    {
      if (strValue.trim().length() > 0)
      {
        bReturn = false;
      }
    }

    return bReturn;
  } // end method empty

  /**
   * String-Array auf null, Lnge 0 oder nur Spaces prfen
   *
   * @param strValue zu prfendes String-Array
   * @return ob String empty (ja/nein)
   */
  public static final boolean empty(String[] strArrValue) {
    if ( strArrValue != null ) {
      if (strArrValue.length > 0) {
        if ( ! empty( strArrValue[ 0 ] ) ) { // hier noch korrigieren, alle Element auf empty prfen
          return false;
        }
      }
    }

    return true;
  } // end method empty

  /**
   * StringBuffer auf leer prfen
   *
   * @param strBuffValue zu prfender StringBuffer
   * @return ob StringBuffer empty (ja/nein)
   */
  public static final boolean empty(StringBuffer strBuffValue)
  {
    boolean bReturn = true;

    if (strBuffValue != null)
    {
      if (strBuffValue.toString().trim().length() > 0)
      {
        bReturn = false;
      }
    }

    return bReturn;
  } // end method empty

  /**
   * Verlngern eines Strings auf die gewnschte Lnge mit Leerzeichen. <br/>
   * Der String wird nicht abgeschnitten, falls er lnger ist als verlangt.
   *
   * @param strPa zu verlngernder String
   * @param iPaLen gewnschte Lnge
   * @return verlngerter String
   */
  public static final String pad( String strPa , int iPaLen ) {
    if ( strPa == null ) {
      strPa = "" ;
    }//end if

    if ( strPa.length() >= iPaLen ) {
        return strPa ;
    }//end if

    StringBuffer sb = new StringBuffer( iPaLen ) ;
    sb.append( strPa ) ;
    while ( sb.length() < iPaLen ) {
        sb.append( ' ' ) ;
    }//end while
    return sb.toString() ;
  }//end method pad


  /**
   * String in Grossbuchtaben umwandeln, mit null-Abfangung
   *
   * @param strValue umzuwandlender String
   * @return umgewandelter String oder null, wenn Parameter-String null war
   */
  public static final String upper(String strValue) {
    if (strValue != null) {
        strValue = strValue.toUpperCase() ;
    }
    return strValue ;
  } // end method upper

  /**
   * String.toLowerCase() mit Abfangung Null-Pointer. <br/>
   *
   * @param strPa umzuwandelnder String
   * @return in Kleinbuchstaben umgewandelter String
   */
  public static final String lower( String strPa ) {
    if ( strPa == null ) {
      return null ;
    }//end if

    return strPa.toLowerCase() ;
  }//end method lower

  /**
   * Verlngern eines Strings auf die gewnschte Lnge durch
   * Einfgen von Leerzeichen am Anfang. <br/>
   * Der String wird nicht abgeschnitten, falls er lnger ist als verlangt.
   *
   * @param strPa zu verlngernder String
   * @param iPaLen gewnschte Lnge
   * @return verlngerter String
   */
  public static final String leftPad( String strPa , int iPaLen ) {
    if ( strPa == null ) {
      strPa = "" ;
    }//end if

    if ( strPa.length() >= iPaLen ) {
        return strPa ;
    }//end if

    StringBuffer sb = new StringBuffer( iPaLen ) ;
    sb.append( strPa ) ;
    while ( sb.length() < iPaLen ) {
        sb.insert( 0 , ' ' ) ;
    }//end while
    return sb.toString() ;
  }//end method leftPad

  /**
   * Filter the specified string for characters that are senstive to
   * HTML interpreters, returning the string with these characters replaced
   * by the corresponding character entities.
   *
   * @param value The string to be filtered and returned
   * @return the filtered String
   */
  public static final String encodeHtml(String value)
  {
    if (value == null)
    {
      return (null);
    }

    char  content[] = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length + 50);

    for (int i = 0; i < content.length; i++)
    {
      switch (content[i])
      {

        case '<':
          result.append("&lt;");

          break;

        case '>':
          result.append("&gt;");

          break;

        case '&':
          result.append("&amp;");

          break;

        case '"':
          result.append("&quot;");

          break;

        default:
          result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method encodeHtml

  /**
   * String in int umwandeln
   *
   * @param numStr String numerisch
   * @return Zahl
   */
  public static final int str2int(String numStr)
  {
    if (empty(numStr))
    {
      return 0;
    }

    numStr = numStr.trim();

    int retInt = 0;

    try
    {
      retInt = (new Integer(numStr)).intValue();
    }
    catch (Exception e)
    {
      ;
    }

    return retInt;
  } // end method str2int

  /**
   * String in Integer-Object umwandeln
   *
   * @param numStr String numerisch
   * @return Intger-Object
   */
  public static final Integer str2Integer( String numStr )
  {
    Integer retInteger = null ;
    retInteger = new Integer( str2int( numStr ) ) ;
    return retInteger;
  } // end method str2Integer

  /**
   * String in int umwandeln mit Tolerierung enthaltener nichtnumerischer Zeichen . <br/>
   * geparst wird bis zum Auftreten nichtnumerischer Zeichen.<br/>
   * Entfernung fhrender Nullen lt sich zuschalten (auskommentierter Code).<br/>
   *
   * @param numStr String numerisch
   * @return int Zahl
   */
  public static final int str2intTolerant(String numStr)
  {
    if (empty(numStr))
    {
      return 0;
    }

    numStr = numStr.trim();

    // Entfernen fhrende Nullen
    // while ( numStr.length() > 1 && numStr.startsWith( "0" ) )
    // {
    // numStr = numStr.substring(1) ;
    // }

    // Entfernen evtl. nichtnumerische Zeichen
    for (int i = 0; i < numStr.length(); i++)
    {
      if (numStr.charAt(i) < '0' || numStr.charAt(i) > '9')
      {

        // nicht numerisches Zeichen
        numStr = numStr.substring(0, i);

        break;
      }
    }

    int retInt = 0;

    try
    {
      retInt = (new Integer(numStr)).intValue();
    }
    catch (Exception e)
    {
      ;
    }

    return retInt;
  } // end method str2intTolerant

  /**
   * berprfung einer Whrungs-Eingabe und Umwandlung in ein double-Wert
   *
   * @param inBetrag Betrag als String, der in ein double umgewandelt werden soll
   * @return double, der aus der Eingabe erzeugt wurde<br>
   * ist inBetrag ein NULL-String, wird Double.NaN zurckgegeben
   * @exception NumberFormatException, falls bei der Umwandlung ein Fehler auftritt
   * @see Double#NaN Double.NaN
   */
  public static double parseDouble(String inBetrag)
    throws NumberFormatException
  {
    double  retDouble;
    String  strTemp;

    if (inBetrag == null)
    {
      retDouble = Double.NaN;
    }
    else
    {
      strTemp = inBetrag.replace(',', '.');
      retDouble = Double.parseDouble(strTemp);
    }

    return retDouble;
  } // end parseDouble

  /**
   * berprfung einer Whrungs-Eingabe und Umwandlung in ein double, aber ohne Werfen NumberFormatException
   *
   * @param inBetrag Betrag als String, der in ein double umgewandelt werden soll
   * @return double, der aus der Eingabe erzeugt wurde<br>
   * ist inBetrag ein NULL-String, wird Double.NaN zurckgegeben
   * @see Double#NaN Double.NaN
   */
  public static double parseDoubleCatched(String inBetrag)
  {
    double  retDouble = 0;

    try
    {
      retDouble = parseDouble(inBetrag);
    }
    catch (NumberFormatException ne)
    {
      ;
    }

    return retDouble;
  } // end parseDouble

  /**
   * Umwandeln String in double-Wert
   *
   * @param parStr String numerisch
   * @return double-Zahl
   */
  public static double str2double(String parStr)
  {
    return parseDoubleCatched(parStr);
  }

  /**
   * Umwandeln String in double-Objekt
   *
   * @param parStr String numerisch
   * @return double-Objekt
   */
  public static Double str2Double(String parStr)
  {
    return new Double(parseDoubleCatched(parStr));
  }

  /**
   * null-String in Leerstring umwandeln
   *
   * @param str eventueller null-String
   * @return nicht null-String
   */
  public static final String notNull(String str)
  {
    if (str == null)
    {
      return "";
    }

    return str;
  } // end method notNull

  /**
   * String in boolean umwandeln. <br/>
   * true: true, TRUE, yes, YES, ja, JA, ok, OK, on, ON, 1 <br/>
   * false: alles andere                                <br/>
   *
   * @param str String mit logischem Wert
   * @return boolean-Wert
   */
  public static final boolean str2bool(String str)
  {
    boolean bReturn = false;

    if ( ! empty( str ) ) {
      str = str.toUpperCase();

      if ( "TRUE".equals( str ) || "YES".equals(str)
          || "JA".equals( str ) || "OK".equals( str )
          || "ON".equals( str ) || "1".equals( str ) )
      {
        bReturn = true;
      }
    }

    return bReturn;
  } // end method str2bool

  /**
   * String in Boolean-Object umwandeln. <br/>
   * true: true, TRUE, yes, YES, ja, JA, ok, OK, on, ON, 1 <br/>
   * false: alles andere                                <br/>
   *
   * @param str String mit logischem Wert
   * @return boolean-Object
   */
  public static final Boolean str2boolObject( String str ) {
    return new Boolean(str2bool(str));
  } // end method str2boolObject

  /**
   * numerischen String in boolean umwandeln. <br/>
   * true:  x >= 1  <br/>
   * false: x <  1  <br/>
   *
   * @param str String numerisch
   * @return boolean-Wert
   */
  public static final boolean numStr2bool(String str)
  {
    boolean bReturn = false;

    if (!empty(str))
    {
      int iTmp = -1;

      try
      {
        iTmp = str2int(str);
      }
      catch (Exception e)
      {

        // Log.fatal (e);
      }

      bReturn = (iTmp > 0);
    }

    return bReturn;
  } // end method str2bool

  /**
   * numerischen String in Boolean-Object umwandeln. <br/>
   * true: true, TRUE, yes, YES, ja, JA, ok, OK, on, ON <br/>
   * false: alles andere                                <br/>
   *
   * @param str String numerisch
   * @return Boolean-Object
   */
  public static final Boolean numStr2boolObject(String str)
  {
    return new Boolean(numStr2bool(str));
  } // end method str2bool

  /**
   * Entferne Hoch-Kommas
   *
   * @param value String mit Hochkommas
   * @return String ohne Hochkommas
   */
  public static final String stripHochKomma(String value)
  {
    if (value == null)
    {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++)
    {
      switch (content[i])
      {

        case (char) 39:
          break;

        default:
          result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripHochKomma

  /**
   * Entfernen Newline (char(10)) und Carriage Return (char(13))
   *
   * @param value String mit Newline (char(10)) und Carriage Return (char(13))
   * @return String ohne Newline (char(10)) und Carriage Return (char(13))
   */
  public static final String stripBreak(String value)
  {
    if (value == null)
    {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++)
    {
      switch (content[i])
      {

        case (char) 10:
          break;

        case (char) 13:
          break;

        default:
          result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripBreak

  /**
   * Entferne Carriage Return (char(13)) und encode Newline (char(10)) zu "\n"
   *
   * @param value String vorher
   * @return String encoded
   */
  public static final String encodeBreak(String value)
  {
    if (value == null)
    {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++)
    {
      switch (content[i])
      {

        case (char) 10:
          result.append((char) 92); // steht fr Backslash
          result.append("n");       // n von \n

          break;

        case (char) 13:
          break;

        default:
          result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripBreak

  /**
   * Entferne Carriage Return (char(13)) und encode Newline (char(10)) zu &lt;br/&gt;
   *
   * @param value String vorher
   * @return String verarbeitet
   */
  public static final String encodeBreakToTag( String value )
  {
    if (value == null)
    {
      return (null);
    }

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++)
    {
      switch (content[i])
      {

        case (char) 10:
          result.append("<br/>");

          break;

        case (char) 13:
          break;

        default:
          result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripBreak

  /**
   * Entfernen Space (32) und Tab (9)
   *
   * @param value String vorher
   * @return String verarbeitet
   */
  public static final String stripTabSpace(String value)
  {
    if (value == null)
    {
      return (null);
    }

    value = value.trim();

    char[]  content = new char[value.length()];

    value.getChars(0, value.length(), content, 0);

    StringBuffer  result = new StringBuffer(content.length);

    for (int i = 0; i < content.length; i++)
    {
      switch (content[i])
      {

        case (char) 32:
          break;

        case (char) 9:
          break;

        default:
          result.append(content[i]);
      }
    }

    return (result.toString());
  } // end method stripTabSpace

  /**
   * ersten Buchstaben im String gro machen, Capitalize
   *
   * @param strPaLower zu bearbeitender String
   * @return kapitalisierter String
   */
  public static final String upper1(String strPaLower)
  {
    String  retString = null;

    if (strPaLower != null)
    {
      if (strPaLower.length() < 2)
      {
        retString = strPaLower.toUpperCase();
      }
      else
      {
        retString = strPaLower.substring(0, 1).toUpperCase();
        retString += strPaLower.substring(1);
      }
    }

    return retString;
  } // end method upper1

  /**
   * aktuelles Datum als String
   *
   * @return aktuelles Datum als String
   */
  public static final String currentDate()
  {
    return date2str(new Date());
  } // end method currentDate

  /**
   * Timestamp fr aktuellen Zeitpunkt erstellen
   *
   * @return current Timestamp as String
   */
  public static final String getTimeStamp()
  {
    return formatTimestamp.format(new Date(System.currentTimeMillis()));
  }

  /**
   * Datum in String umwandeln
   *
   * @param dateValue Datum
   * @return Datums-String
   */
  public static final String date2str(Date dateValue) {
    return formatDate.format(dateValue);
  } // end method date2str

  /**
   * String im Format dd.MM.yyyy in Datum umwandeln
   *
   * @param strValue String mit Datum im Format dd.MM.yyyy
   * @return Datum oder null, wenn Datum ungltig
   */
  public static final Date str2date(String strValue) {
    return parseDateCatched(strValue);
  } // end method str2date

  /**
   * String im Format dd.MM.yyyy in SQL-Datum umwandeln
   *
   * @param strValue String mit Datum im Format dd.MM.yyyy
   * @return SQL-Datum
   */
  public static final java.sql.Date str2sqldate(String strValue)
  {
    java.util.Date  dateTmp = str2date(strValue);

    if (dateTmp != null)
    {
      return new java.sql.Date(dateTmp.getTime());
    }

    return null;
  } // end method str2sqldate

  /**
   * Vergleich zweier Strings mit Nullpointer-Behandlung
   *
   * @param obj1 erster String
   * @param obj2 zweiter String
   * @return ob gleich
   */
  public static final boolean stringEqual( String obj1 , String obj2 ) {
    if (obj1 == null && obj2 == null)
    {
      return true;
    }

    if (obj1 != null)
    {
      if (obj1.equals(obj2))
      {
        return true;
      }
    }

    return false;
  } // end method stringEqual

  /**
   * Vergleich zweier Double mit Nullpointer-Behandlung
   *
   * @param obj1 erstes Double
   * @param obj2 zweites Double
   * @return ob gleich
   */
  public static final boolean doubleEqual(Double obj1, Double obj2)
  {
    if (obj1 == null && obj2 == null)
    {
      return true;
    }

    if (obj1 != null)
    {
      if (obj1.equals(obj2))
      {
        return true;
      }
    }

    return false;
  } // end method doubleEqual

  /**
   * Vergleich zweier Objekte mit Nullpointer-Behandlung
   *
   * @param obj1 erstes Objekt
   * @param obj2 zweites Objekt
   * @return ob gleich
   */
  public static final boolean objectEqual(Object obj1, Object obj2) {
    if (obj1 == null && obj2 == null)
    {
      return true;
    }

    if (obj1 != null && obj2 != null)
    {
      if (obj1.getClass().isInstance(obj2))
      {
        if (obj1.equals(obj2))
        {
          return true;
        }
      }
    }

    return false;
  } // end method objectEqual

  /**
   * berprfung einer Datums-Eingabe und ggf. Umwandlung in ein Date-Objekt.
   *
   * @param inDatum Datum als String, das in ein Date-Objekt umgewandelt werden soll
   *
   * @return retDatum Datum-Objekt, das aus der Eingabe erzeugt wurde
   *
   * @exception ParseException: Falls der Eingabestring nicht in ein Datum
   * umgewandelt werden kann, wird eine ParseException geworfen
   */
  public static Date parseDate(String inDatum)
    throws ParseException
  {
    Date              retDatum;
    String            strDatum = "";
    String            strDay;
    String            strMonth;
    String            strYear;
    int               indexDay;
    int               indexMonth;
    GregorianCalendar cal;
    int               intError = -1;
    ParseException    e = new ParseException("Ecis: Not a valid Date", intError);

    ParsePosition     pos = new ParsePosition(0);

    if (inDatum == null)
    {
      retDatum = null;
    }
    else
    {
      strDatum = inDatum.trim();

      try
      {
        indexDay = strDatum.indexOf(".");
        indexMonth = strDatum.indexOf(".", indexDay + 1);
        strDay = strDatum.substring(0, indexDay);
        strMonth = strDatum.substring(indexDay + 1, indexMonth);
        strYear = strDatum.substring(indexMonth + 1);

        if (strYear.length() != 4)
        {
          throw e;
        }

        if ((str2int(strMonth) > 0) && (str2int(strMonth) < 13))
        {
          cal = new GregorianCalendar(str2int(strYear), (str2int(strMonth) - 1), 1);

          if ((str2int(strDay) < 1) || (str2int(strDay) > cal.getActualMaximum(Calendar.DAY_OF_MONTH)))
          {
            throw e;
          }
        }
        else
        {
          throw e;
        }
      }
      catch (Exception ex)
      {
        throw e;
      }

      formatDate.setLenient(false);

      retDatum = formatDate.parse(strDatum, pos);

      if (retDatum == null)
      {
        throw e;
      }
    }

    return retDatum;
  } // end parseDate

  /**
   * berprfung einer Datums-Eingabe und ggf. Umwandlung in ein Date-Objekt.
   *
   * @param inDatum Datum als String, das in ein Date-Objekt umgewandelt werden soll
   * @return retDatum Datum-Objekt, das aus der Eingabe erzeugt wurde
   */
  public static Date parseDateCatched(String inDatum) {
    Date  retDatum = null;

    try {
      retDatum = parseDate(inDatum);
    }
    catch (ParseException pe) {
      // pe.printStackTrace() ;
    }
    return retDatum;
  } // end parseDateCatched

  /**
   * Datum auf Gltigkeit prfen
   *
   * @param testString
   * @return gltig (ja/nein)
   */
  public static boolean testDateString(String testString) {
    Date  testDate = parseDateCatched( testString ) ;
    if (testDate == null) {
      return false;
    }
    return true;
  } // end method testDateString

  /**
   * Zurckgeben der iLength Zeichen eines Strings ab dem linken Rand. <br/>
   * Mit Behandlung Nullpointer und Grenberlauf. <br/>
   * Wenn String krzer als iLength, werden so viele Zeichen wie vorhanden zurckgegeben. <br/>
   *
   * @param parString Original-String
   * @param iLength Anzahl Zeichen
   * @return String links
   */
  public static String left(String parString, int iLength)
  {
    String  retString = null;

    if (parString != null)
    {
      if (parString.length() > 0)
      {
        iLength = Math.min(iLength, parString.length());
        retString = parString.substring(0, iLength);
      }
      else
      {
        retString = parString;
      }
    }

    return retString;
  } // end method left

  /**
   * list(PrintWriter)-Methode eines Properties-Objekts auf einen String umgeleitet
   *
   * @param prop Properties
   * @return list-String
   */
  public static String listToString(Properties prop)
  {
    String  retString = "";

    if (prop != null)
    {
      StringWriter  strWriter = new StringWriter();
      PrintWriter   pWriter = new PrintWriter(strWriter);

      prop.list(pWriter);

      retString = strWriter.toString();
    }

    return retString;
  } // end method listToString

  /**
   * Textdatei zeilenweise in ein String-Array einziehen . <br/>
   * Jede Zeile bildet einen String im String-Array.<br/>
   *
   * @param fileName Dateiname
   * @return String-Array
   */
  public static String[] fileToStrArr(String fileName)
  {
    if (fileName == null)
    {
      return null;
    }

    return fileToStrArr(new File(fileName));
  } // end method fileToStrArr

  /**
   * Textdatei zeilenweise in ein String-Array einziehen . <br/>
   * Jede Zeile bildet einen String im String-Array.<br/>
   *
   * @param inFile File-Object
   * @return String-Array
   */
  public static String[] fileToStrArr( File inFile ) {
    if (inFile == null)
    {
      return null;
    }

    try
    {
      // BufferedReader zum zeilenweisen Lesen
      FileReader      defFileReader = new FileReader(inFile);
      BufferedReader  defBufferedReader = new BufferedReader(defFileReader);
      ArrayList       strArrayList = new ArrayList();
      String          readLine = null;

      while ((readLine = defBufferedReader.readLine()) != null) // bis zum Ende der Datei
      {
        //readLine = readLine.trim();
        strArrayList.add(readLine);
      } // end while

      String[]  retArr = arrayListToStrArr(strArrayList);

      return retArr;
    }
    catch (Exception e) {

      //Log.fatal( e ) ;
      //throw new RuntimeException( Log.getMessageAndStackTrace( e ) ) ;
      System.err.println( e.getMessage() );
      e.printStackTrace();
      throw new RuntimeException( e.getMessage() );
    }
    //return null ;
  } // end method fileToStrArr

  /**
   * ArrayList mit Strings in String[] umwandeln . <br/>
   * Verwendet in {@link #fileToStrArr}.<br/>
   *
   * @param strArrayList ArrayList mit Strings
   * @return String[]
   * @exception ClassCastException wenn Element kein String
   */
  public static String[] arrayListToStrArr( ArrayList strArrayList ) {
    if ( strArrayList == null || strArrayList.isEmpty () )
    {
      return new String[0];
    }

    String[]  retArr = new String[strArrayList.size ()];

    for (int i = 0; i < strArrayList.size (); i++)
    {
      retArr[i] = (String) strArrayList.get (i);
    }

    return retArr;
  } // end method arrayListToStrArr

  /**
   * Teilstring von String links abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutStr abzuschneidender String
   * @return abgeschnittener String
   */
  public static String cutLeft( String srcStr , String cutStr ) {
    if ( srcStr == null || cutStr == null ) {
      return srcStr;
    }

    String  retStr = srcStr;

    if (retStr.length() >= cutStr.length() && retStr.startsWith( cutStr ) ) {
      retStr = retStr.substring(cutStr.length()); // cutStr abschneiden
    }

    return retStr;
  }//end method cutLeft

  /**
   * Teilstring von String rechts abschneiden, wenn mglich
   *
   * @param srcStr Original-String
   * @param cutStr abzuschneidender String
   * @return abgeschnittener String
   */
  public static String cutRight( String srcStr , String cutStr ) {
    if (srcStr == null || cutStr == null) {
      return srcStr;
    }

    String  retStr = srcStr;

    if ( retStr.length() >= cutStr.length() && retStr.endsWith( cutStr ) ) {
      retStr = retStr.substring(0, retStr.length() - cutStr.length());  // cutStr abschneiden
    }

    return retStr;
  }//end method cutRight

  /**
   * Speichern eines String in einer Datei ohne weitere Informationen
   *
   * @param outStr   auszugebender String
   * @param fileName Name der Datei, in welcher der String gespeichert werden soll
   * @return         ob erfolgreich
   */
  public static boolean str2file( String outStr , String fileName ) {
    return str2file(outStr, new File(fileName));
  } // end method str2file

  /**
   * Speichern eines String in einer Datei ohne weitere Informationen
   *
   * @param outStr   auszugebender String
   * @param outFile  Datei, in welcher der String gespeichert werden soll
   * @return         ob erfolgreich
   */
  public static boolean str2file( String outStr , File outFile ) {
    boolean bReturn = false;

    try {
      if ( outFile.exists() ) {
        outFile.delete();
      }

/*
//System.out.println( "hier noch Verzeichnis anlegen einbauen" ) ;
      FileUtil.createDirectoryForFile( outFile ) ;
System.out.println( outFile.getAbsolutePath() );
      outFile.createNewFile();
*/
      //Datei erzeugen mit Lschen evtl. vorhandene alte Datei und erzeugen Verzeichniss
      createNewFile( outFile );

      FileWriter outFileWriter = new FileWriter( outFile ) ;

      outFileWriter.write(outStr);
      outFileWriter.close();

      bReturn = true;
    }
    catch ( Exception e ) {
      //Log.fatal(e);
      //Log.debug(e.getMessage());
      //e.printStackTrace();
      System.err.println( e.getMessage() );
      e.printStackTrace();
    }

    return bReturn;
  } // end method str2file


  /**
   * Einlesen einer Datei in einen String
   *
   * @param  fileName Name der einzulesenden Datei
   * @return String mit Datei-Inhalt
   */
  public static String file2str(String fileName)
  {
    return file2str(new File(fileName));
  } // end method file2str

  /**
   * Einlesen einer Datei in einen String
   *
   * @param inFile einzulesende Datei
   * @return String mit Datei-Inhalt
   */
  public static String file2str(File inFile)
  {
    String  strReturn = null;

    try {
      StringBuffer    strBuff = new StringBuffer( 1024 * 16 );

      // BufferedReader zum zeilenweisen Lesen
      FileReader      defFileReader = new FileReader(inFile);
      BufferedReader  defBufferedReader = new BufferedReader(defFileReader);
      String          readLine = null;

      while ((readLine = defBufferedReader.readLine()) != null ) { // bis zum Ende der Datei
        strBuff.append(readLine + "\n");
      } // end while

      strReturn = strBuff.toString();
    }
    catch ( Exception e ) {
      //Log.fatal(e);
      //Log.debug(e.getMessage());
      System.err.println( e.getMessage() );
      e.printStackTrace();
    }
    return strReturn;
  }//end method file2str


    /**
     * Umwandeln int-Wert in String mit Angabe Anzahl Stellen
     *
     * @param int-Wert
     * @param Anzahl Stellen
     *
     * @return numerischer String, rechtsbndig
     */
    public static String str( int iPaValue , int iPaDigits ) {
        return leftPad( "" + iPaValue , iPaDigits ) ;
    }//end method str(int)

    /**
     * Umwandeln Integer-Objekt in String mit Angabe Anzahl Stellen. <br/>
     * Bei bergabe null-Pointer wird der Wert 0 angenommen
     *
     * @param iPaValue Integer-Objekt
     * @param iPaDigits Anzahl Stellen
     *
     * @return numerischer String, rechtsbndig
     */
    public static String str( Integer iPaValue , int iPaDigits ) {
        int iTmp = 0 ;

        if ( iPaValue != null ) {
            iTmp = iPaValue.intValue() ;
        }//end if

        return leftPad( "" + iTmp , iPaDigits ) ;
    }//end method str(Integer)

  /**
   * int-Wert in String mit fhrenden Nullen umwandeln
   */
  public static String strzero( int iPaValue , int iPaDigits ) {
    StringBuffer sb = new StringBuffer( iPaDigits ) ;
    sb.append( "" + iPaValue ) ;
    while ( sb.length() < iPaDigits ) {
      sb.insert( 0 , '0' ) ;
    }//end while
    return sb.toString() ;
  }//end method strzero

    /**
     * Zurckliefern String-Teil vor dem Bindestrich aus einem String der einen Bindestrich enthlt. <br/>
     *
     * @param strPaHyphen String mit Bindestrich
     *
     * @return String-Teil vor Bindestrich
     */
    public static String preHyphen( String strPaHyphen ) {
//Log.debug( strPaHyphen ) ;
        StringTokenizer tok = new StringTokenizer( strPaHyphen , "-" ) ;
        return tok.nextToken() ;
    }//end method preHyphen

    /**
     * Zurckliefern String-Teil nach dem Bindestrich aus einem String der einen Bindestrich enthlt. <br/>
     *
     * @param strPaHyphen String mit Bindestrich
     *
     * @return String-Teil nach Bindestrich
     */
    public static String postHyphen( String strPaHyphen ) {
//Log.debug( strPaHyphen ) ;
        StringTokenizer tok = new StringTokenizer( strPaHyphen , "-" ) ;
        tok.nextToken() ;
        return tok.nextToken() ;
    }//end method preHyphen

    /**
     * Zurckliefern String-Teil vor dem Punkt aus einem String der einen Punkt enthlt. <br/>
     *
     * @param strPaHyphen String mit Punkt
     *
     * @return String-Teil vor Punkt
     */
    public static String prePoint( String strPaPoint ) {
//Log.debug( strPaPoint ) ;
        StringTokenizer tok = new StringTokenizer( strPaPoint , "." ) ;
        return tok.nextToken() ;
    }//end method prePoint

    /**
     * Zurckliefern String-Teil nach dem Punkt aus einem String der einen Punkt enthlt. <br/>
     *
     * @param strPaPoint String mit Punkt
     *
     * @return String-Teil nach Punkt
     */
    public static String postPoint( String strPaPoint ) {
//Log.debug( strPaPoint ) ;
        StringTokenizer tok = new StringTokenizer( strPaPoint , "." ) ;
        tok.nextToken() ;
        return tok.nextToken() ;
    }//end method prePoint

    /**
     * Zurckliefern String-Teil vor dem Nummern-Symbol # aus einem String der einen Nummern-Symbol # enthlt. <br/>
     *
     * @param strPaHyphen String mit Nummern-Symbol #
     *
     * @return String-Teil vor Nummern-Symbol #
     */
    public static String preNumSign( String strPaNumSign ) {
//Log.debug( strPaNumSign ) ;
        StringTokenizer tok = new StringTokenizer( strPaNumSign , "#" ) ;
        return tok.nextToken() ;
    }//end method preNumSign

    /**
     * Zurckliefern String-Teil nach dem Nummern-Symbol # aus einem String der einen Nummern-Symbol # enthlt. <br/>
     *
     * @param strPaNumSign String mit Nummern-Symbol #
     *
     * @return String-Teil nach Nummern-Symbol #
     */
    public static String postNumSign( String strPaNumSign ) {
//Log.debug( strPaNumSign ) ;
        StringTokenizer tok = new StringTokenizer( strPaNumSign , "#" ) ;
        tok.nextToken() ;
        return tok.nextToken() ;
    }//end method preNumSign

    /**
     * Wiederholt einen String n-Mal
     *
     * @param strPaSingle zu wiederholender String
     * @param iPaCount    Anzahl Wiederholungen
     */
    public static String repeat( String strPaSingle , int iPaCount ) {
        StringBuffer sb = new StringBuffer( iPaCount ) ;
        for ( int i = 0 ; i < iPaCount ; i++ ) {
            sb.append( strPaSingle ) ;
        }
        return sb.toString() ;
    }//end method repeat

    /**
     * Prfen, ob ein String in einem anderen String enthalten ist mit Nullpointer-Abfangung
     *
     * @param testStr zu durchsuchender String
     * @param searchStr gesuchter String
     *
     * @return ob enthalten
     */
    public static boolean contents( String testStr , String searchStr ) {
      boolean bRet = false ;
      if ( testStr != null ) {
        bRet = ( testStr.indexOf( searchStr ) > -1 ) ;
      }//end if
      return bRet ;
    }//end method contents

    /**
     * Prfen, ob ein Zeichen in einem anderen String enthalten ist mit Nullpointer-Abfangung
     *
     * @param testStr zu durchsuchender String
     * @param searchChr gesuchtes Zeichen
     *
     * @return ob enthalten
     */
    public static boolean contents( String testStr , char searchChr ) {
      boolean bRet = false ;
      if ( testStr != null ) {
        bRet = ( testStr.indexOf( searchChr ) > -1 ) ;
      }//end if
      return bRet ;
    }//end method contents

  /**
   * Trimmen aller Array-Elemente
   */
  public static String[] trim( String[] strArrPa ) {
    if ( strArrPa == null ) {
      return null;//Garbage-In Garbage-Out
    }//end if

    if ( strArrPa.length < 1 ) {
      return strArrPa;
    }//end if

    for ( int i = 0 ; i < strArrPa.length ; i++ ) {
      if ( strArrPa[ i ] != null ) {
        strArrPa[ i ] = strArrPa[ i ].trim() ;
      }//end if
    }//end for
    return strArrPa;
  }//end method trim

  /**
   * Trimmen aller ArrayList-Elemente
   */
  public static ArrayList trim( ArrayList arrListPa ) {
    if ( arrListPa == null ) {
      return null;//Garbage-In Garbage-Out
    }//end if

    if ( arrListPa.size() < 1 ) {
      return arrListPa;
    }//end if

    for ( int i = 0 ; i < arrListPa.size() ; i++ ) {
      if ( arrListPa.get( i ) != null ) {
        arrListPa.set( i , ( (String) arrListPa.get( i ) ).trim() ) ;
      }//end if
    }//end for
    return arrListPa;
  }//end method trim

  /**
   * Entfernen Kommentarzeilen, die mit // beginnen
   */
  public static String[] stripRemarksFromStrArr( String[] strArrPa ) {
    if ( strArrPa == null ) {
      return null;//Garbage-In Garbage-Out
    }//end if

    if ( strArrPa.length < 1 ) {
      return strArrPa;
    }//end if

    ArrayList arrList = new ArrayList();

    for ( int i = 0 ; i < strArrPa.length ; i++ ) {
      String strTmp = strArrPa[ i ].trim() ;
      if ( ! strTmp.startsWith( "//" ) ) {
        arrList.add( strTmp ) ;
      }//end if
    }//end for
    return arrayListToStrArr( arrList ) ;
  }//end method stripRemarksFromStrArr

  /**
   * Anfgen eines Strings an ein String-Array
   *
   * @param originalArr Original-String[]
   * @param addStr anzuhngener String
   * @return Ergebnis-String[]
   */
  public static String[] arrAdd(String[] originalArr, String addStr) {
    if (originalArr == null) {
      originalArr = new String[0];
    }
    String[]  newArr = new String[originalArr.length + 1];
    for (int i = 0; i < originalArr.length; i++) {
      newArr[i] = originalArr[i];
    }
    newArr[newArr.length - 1] = addStr;
    return newArr;
  } // end method arrAdd

  /**
   * links beginnend bis zum ersten Leerzeichen lesen
   */
  public static String leftToSpace( String strPa ) {
    if ( strPa == null ) {
      return null;//Garbage-In Garbage-Out
    }//end if

    StringBuffer retSb = new StringBuffer();

    int pos = 0 ; // Position im String

    //ber beginnende Leerzeichen laufen
    for (  ; pos < strPa.length() && strPa.charAt( pos ) == ' ' ; pos++ ) ;

    //bis zum nchsten Leerzeichen laufen
    for (  ; pos < strPa.length() && strPa.charAt( pos ) != ' ' ; pos++ ) {
      retSb.append( strPa.charAt( pos ) ) ;
    }//end for

    return retSb.toString();
  }//end method leftToSpace

  /**
   *
   */
  public static ArrayList commaSepArrList( String strPa ) {
    if ( strPa == null ) {
      return null;//Garbage-In Garbage-Out
    }//end if

    StringTokenizer strToknzr = new StringTokenizer( strPa , "," );

    ArrayList arrList = new ArrayList();

    while ( strToknzr.hasMoreTokens() ) {
      arrList.add( strToknzr.nextToken().trim() );
    }//end while

    return arrList ;
  }//end method commaSepArrList

  /**
   * Returns a String with all occurrences of <code>from</code>
   * within <code>orig</code> replaced with <code>to</code>. If
   * <code>orig</code> contains no occurrences of
   * <code>from</code>, or if <code>from</code> is equal to
   * <code>to</code>, <code>orig</code> itself is returned rather
   * than a copy being made. None of the parameters should be
   * null.
   *
   * @param orig the original String. Must not be null.
   * @param from the String to replace within <code>orig</code>.
   * Must not be null.
   * @param to the String to replace <code>from</code> with. Must
   * not be null.
   *
   * @returns a version of <code>orig</code> with all occurrences
   * of <code>from</code> being replaced with <code>to</code>.
   *
   * @throws IllegalArgumentException if <code>from</code> is empty
   */
  public static String replace(String orig, String from, String to)
  {
    int fromLength = from.length();

    if (fromLength == 0)
    {
      throw new IllegalArgumentException("String to be replaced must not be empty");
    }

    int start = orig.indexOf(from);

    if (start == -1)
    {
      return orig;
    }
    boolean       greaterLength = (to.length() >= fromLength);
    StringBuffer  buffer;

    // If the "to" parameter is longer than (or
    // as long as) "from", the final length will
    // be at least as large
    if (greaterLength)
    {
      if (from.equals(to))
      {
        return orig;
      }
      buffer = new StringBuffer(orig.length());
    }
    else
    {
      buffer = new StringBuffer();
    }

    char[]  origChars = orig.toCharArray();
    int     copyFrom = 0;

    while (start != -1)
    {
      buffer.append(origChars, copyFrom, start - copyFrom);
      buffer.append(to);

      copyFrom = start + fromLength;
      start = orig.indexOf(from, copyFrom);
    }
    buffer.append(origChars, copyFrom, origChars.length - copyFrom);
    return buffer.toString();
  }

  /**
   * Erzeugen einer neuen Datei . <br/>
   * Eine eventuell vorhandene gleichnamige Datei wird vorher gelscht.<br/>
   * Falls notwendig, wird das Verzeichnis fr die Datei angelegt.<br/>
   *
   * @param filePa anzulegende Datei
   * @throws IOException weiterwerfen Exception vom Erzeugen der Datei
   */
  private static void createNewFile(File filePa)
      throws IOException {
    if (filePa.exists()) {
      filePa.delete();
    }

    try {
      filePa.createNewFile();
    } catch (IOException ioe) {
      // Versuch Verzeichnis anzulegen
      (new File(filePa.getParent())).mkdirs();
      // zweiter Versuch
      filePa.createNewFile();
    }// end catch
  }// end method

  /**
   * Datei kopieren
   *
   * @param source Name mit Pfad der zu kopierenden Datei
   * @param dest   Name mit Pfad des Kopierzieles
   */
  private static void copyFile(String source, String dest) {
    final int PUFF_SIZE = 32000; // Puffergroesse kann entsprechend Speicher angepasst werden
    byte[] puff;
    long lastModified;

    try {
      FileInputStream fis = new FileInputStream(source);
      File fo = new File(dest);

      if (fo.exists()) {
        if (!fo.delete()) { // Zieldatei erst mal loeschen
          String s = new String("copyFile " + source + " " + dest + " not available");

          System.err.println(s);
          throw new IOException(s);
        }
      }

      // fo.createNewFile();
      createNewFile(fo);

      FileOutputStream fos = new FileOutputStream(fo);

      while (fis.available() > 0) {
        if (fis.available() >= PUFF_SIZE) {
          puff = new byte[PUFF_SIZE];
        }
        else {
          puff = new byte[fis.available()];
        }
        fis.read(puff);
        fos.write(puff);
      }
      fis.close();
      lastModified = (new File(source)).lastModified();
      fos.close();
      fo.setLastModified(lastModified); // beim Kopieren das nderungsdatum erhalten
    } catch (IOException ioX) {
      System.err.println(ioX.getMessage());
      System.err.println("copyFile " + source + " " + dest + " not available");
    }
  }// end method copyFile

    /**
     * Abfrage eines Zeichen von einem String mit Abfangung
     * Exception bei falscher Position.
     * 
     * @param pStr abzufragender String
     * @param pPos Zeichenposition
     * @return abgefragtes Zeichen oder chr(0), wenn Position nicht ok
     */
    public static char charAt(String pStr , int pPos)
    {
        try
        {
            return pStr.charAt(pPos);
        }
        catch (Exception e)
        {
            return (char) 0;
        }
    }
        

} // end class StringUtil
